@@ -317,6 +317,22 @@ void MediaObject::setSource(const MediaSource &source)
     setSourceInternal(source, HardSwitch);
 }
 
+static QByteArray mrlEncode(QByteArray mrl)
+{
+    for (int i = 0; i < mrl.size(); ++i) {
+        const unsigned char c = static_cast<unsigned char>(mrl.at(i));
+        // we assume that the other invalid characters
+        // are already escaped due to the call to QUrl.toEncoded()
+        if (c == '#') {
+            char enc[4];
+            qsnprintf(enc, 4, "%%%02X", c);
+            mrl = mrl.left(i) + QByteArray(enc, 3) + mrl.mid(i + 1);
+            i += 2;
+        }
+    }
+    return mrl;
+}
+
 void MediaObject::setSourceInternal(const MediaSource &source, HowToSetTheUrl how)
 {
     //debug() << Q_FUNC_INFO;
@@ -340,13 +356,18 @@ void MediaObject::setSourceInternal(const MediaSource &source, HowToSetTheUrl ho
             m_stream->setError(Phonon::NormalError, tr("Cannot open media data at '<i>%1</i>'").arg(source.url().toString(QUrl::RemovePassword)));
             return;
         }
-        switch (how) {
-        case GaplessSwitch:
-            m_stream->gaplessSwitchTo(source.url());
-            break;
-        case HardSwitch:
-            m_stream->setUrl(source.url());
-            break;
+        {
+            const QByteArray &mrl = (source.url().scheme() == QLatin1String("") ?
+                    "file:/" + mrlEncode (source.url().toEncoded()) :
+                    mrlEncode (source.url().toEncoded()));
+            switch (how) {
+                case GaplessSwitch:
+                    m_stream->gaplessSwitchTo(mrl);
+                    break;
+                case HardSwitch:
+                    m_stream->setMrl(mrl);
+                    break;
+            }
         }
         break;
     case MediaSource::Disc:
@@ -411,7 +432,7 @@ void MediaObject::setSourceInternal(const MediaSource &source, HowToSetTheUrl ho
 //X void MediaObject::openMedia(Phonon::MediaObject::Media m, const QString &mediaDevice)
 //X {
 //X     m_titles.clear();
-//X 
+//X
 //X }
 
 QByteArray MediaObject::autoplayMrlsToTitles(const char *plugin, const char *defaultMrl)
@@ -660,7 +681,7 @@ void MediaObject::upstreamEvent(Event *e)
         break;
     case Event::RequestSnapshot:
         // postEvent takes ownership of the event and will delete it when done
-        QCoreApplication::postEvent(m_stream, new Event(e->type()));
+        QCoreApplication::postEvent(m_stream, copyEvent(static_cast<RequestSnapshotEvent*>(e)));
         break;
     case Event::SetParam:
         // postEvent takes ownership of the event and will delete it when done
